@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.15';
+$VERSION = '1.16';
 
 sub HandleStruct($$;$$$$);
 
@@ -44,7 +44,8 @@ my %uidInfo = (
     NOTES => q{
         The following tags are extracted from Matroska multimedia container files. 
         This container format is used by file types such as MKA, MKV, MKS and WEBM. 
-        For speed, by default ExifTool extracts tags only up to the first Cluster.
+        For speed, by default ExifTool extracts tags only up to the first Cluster
+        unless a Seek element specifies the position of a Tags element after this.
         However, the L<Verbose|../ExifTool.html#Verbose> (-v) and L<Unknown|../ExifTool.html#Unknown> = 2 (-U) options force processing of
         Cluster data, and the L<ExtractEmbedded|../ExifTool.html#ExtractEmbedded> (-ee) option skips over Clusters to
         read subsequent tags.  See
@@ -112,8 +113,25 @@ my %uidInfo = (
         Name => 'Seek',
         SubDirectory => { TagTable => 'Image::ExifTool::Matroska::Main' },
     },
-    0x13ab => { Name => 'SeekID',           Binary => 1, Unknown => 1 },
-    0x13ac => { Name => 'SeekPosition',     Format => 'unsigned', Unknown => 1 },
+    0x13ab => {
+        Name => 'SeekID',
+        Unknown => 1,
+        SeekInfo => 'ID',       # save seek ID's
+        # (note: converted from VInt internally)
+        PrintConv => q{
+            my $tagInfo = $Image::ExifTool::Matroska::Main{$val};
+            $val = sprintf('0x%x', $val);
+            $val .= " ($$tagInfo{Name})" if ref $tagInfo eq 'HASH' and $$tagInfo{Name};
+            return $val;
+        },
+    },
+    0x13ac => {
+        Name => 'SeekPosition',
+        Format => 'unsigned',
+        Unknown => 1,
+        SeekInfo => 'Position', # save seek positions
+        RawConv => '$val + $$self{SeekHeadOffset}',
+    },
 #
 # Segment Info
 #
@@ -350,8 +368,9 @@ my %uidInfo = (
         Name => 'VideoScanType',
         Format => 'unsigned',
         PrintConv => {
-            0 => 'Progressive',
+            0 => 'Undetermined',
             1 => 'Interlaced',
+            2 => 'Progressive',
         },
     },
     0x13b8 => {
@@ -893,6 +912,7 @@ sub HandleStruct($$;$$$$)
 # Inputs: 0) data buffer, 1) position in data
 # Returns: integer value and updates position, -1 for unknown/reserved value,
 #          or undef if no data left
+# Notes: Increments position pointer
 sub GetVInt($$)
 {
     return undef if $_[1] >= length $_[0];
@@ -929,7 +949,7 @@ sub ProcessMKV($$)
 {
     my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
-    my ($buff, $buf2, @dirEnd, $trackIndent, %trackTypes, $struct);
+    my ($buff, $buf2, @dirEnd, $trackIndent, %trackTypes, $struct, %seekInfo, %seek);
 
     $raf->Read($buff, 4) == 4 or return 0;
     return 0 unless $buff =~ /^\x1a\x45\xdf\xa3/;
@@ -952,6 +972,7 @@ sub ProcessMKV($$)
     my $processAll = ($verbose or $et->Options('Unknown') > 1) ? 2 : 0;
     ++$processAll if $et->Options('ExtractEmbedded');
     $$et{TrackTypes} = \%trackTypes;  # store Track types reference
+    $$et{SeekHeadOffset} = 0;
     my $oldIndent = $$et{INDENT};
     my $chapterNum = 0;
     my $dirName = 'MKV';
@@ -960,6 +981,16 @@ sub ProcessMKV($$)
     for (;;) {
         while (@dirEnd) {
             if ($pos + $dataPos >= $dirEnd[-1][0]) {
+                if ($dirEnd[-1][1] eq 'Seek') {
+                    # save seek info
+                    if (defined $seekInfo{ID} and defined $seekInfo{Position}) {
+                        my $seekTag = $$tagTablePtr{$seekInfo{ID}};
+                        if (ref $seekTag eq 'HASH' and $$seekTag{Name}) {
+                            $seek{$$seekTag{Name}} = $seekInfo{Position} + $$et{SeekHeadOffset};
+                        }
+                    }
+                    undef %seekInfo;
+                }
                 pop @dirEnd;
                 if ($struct) {
                     if (@dirEnd and $dirEnd[-1][2]) {
@@ -993,9 +1024,10 @@ sub ProcessMKV($$)
         }
         my $tag = GetVInt($buff, $pos);
         last unless defined $tag and $tag >= 0;
+        $$et{SeekHeadOffset} = $pos if $tag == 0x14d9b74;   # save offset of seek head
         my $size = GetVInt($buff, $pos);
         last unless defined $size;
-        my $unknownSize;
+        my ($unknownSize, $seekInfoOnly);
         $size < 0 and $unknownSize = 1, $size = 1e20;
         if (@dirEnd and $pos + $dataPos + $size > $dirEnd[-1][0]) {
             $et->Warn("Invalid or corrupted $dirEnd[-1][1] master element");
@@ -1010,14 +1042,28 @@ sub ProcessMKV($$)
             next;
         }
         my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
-        # just fall through into the contained EBML elements
+        if (not $tagInfo and ref $$tagTablePtr{$tag} eq 'HASH' and $$tagTablePtr{$tag}{SeekInfo}) {
+            $tagInfo = $$tagTablePtr{$tag};
+            $seekInfoOnly = 1;
+        }
         if ($tagInfo) {
             if ($$tagInfo{SubDirectory}) {
                 # stop processing at first cluster unless we are using -v -U or -ee
+                # or there are Tags after this
                 if ($$tagInfo{Name} eq 'Cluster' and $processAll < 2) {
-                    last unless $processAll;
+                    # jump to Tags if possible
+                    unless ($processAll) {
+                        if ($seek{Tags} and $seek{Tags} > $pos + $dataPos and $raf->Seek($seek{Tags},0)) {
+                            $buff = '';
+                            $dataPos = $seek{Tags};
+                            $pos = $dataLen = 0;
+                            next;
+                        }
+                        last;
+                    }
                     undef $tagInfo; # just skip the Cluster when -ee is used
                 } else {
+                    # just fall through into the contained EBML elements
                     $$et{INDENT} .= '| ';
                     $et->VerboseDir($$tagTablePtr{$tag}{Name}, undef, $size);
                     $dirName = $$tagInfo{Name};
@@ -1118,6 +1164,11 @@ sub ProcessMKV($$)
         if ($$tagInfo{NoSave} or $struct) {
             $et->VerboseInfo($tag, $tagInfo, Value => $val, %parms) if $verbose;
             $$struct{$$tagInfo{Name}} = $val if $struct;
+        } elsif ($$tagInfo{SeekInfo}) {
+            my $p = $pos;
+            $val = GetVInt($buff, $p) unless defined $val;
+            $seekInfo{$$tagInfo{SeekInfo}} = $val;
+            $et->HandleTag($tagTablePtr, $tag, $val, %parms) unless $seekInfoOnly;
         } else {
             $et->HandleTag($tagTablePtr, $tag, $val, %parms);
         }
